@@ -1,10 +1,28 @@
-import livetest
-
-from base64 import b64encode
-
 from robot.output import LOGGER
 
+from base64 import b64encode
+from functools import wraps
+
+import livetest
+import json
+import jsonpointer
+
+
+def _with_json(f):
+    @wraps(f)
+    def wrapper(self, json_string, *args, **kwargs):
+        return json.dumps(
+          f(self, json.loads(json_string), *args, **kwargs))
+    return wrapper
+
 class HTTP:
+    """
+    HttpLibrary for Robot Framework
+
+    *JSON*
+
+    The JSON related keywords use JSON Pointer. To learn more about JSON Pointer, go to http://tools.ietf.org/html/draft-pbryan-zyp-json-pointer-00.
+    """
 
     # internal
 
@@ -106,14 +124,14 @@ class HTTP:
         Fails if the response status code of the previous request was >= 400
         """
         assert int(self.response.status[0:3]) < 400, \
-               'Response should have succeeded, but was "%s"' % self.response.status
+               'Response should have succeeded, but was "%s".' % self.response.status
 
     def response_should_not_succeed(self):
         """
         Fails if the response status code of the previous request was < 400
         """
         assert int(self.response.status[0:3]) > 399, \
-               'Response should have been success, but was "%s"' % self.response.status
+               'Response should not have succeeded, but was "%s".' % self.response.status
 
     def response_status_code_should_equal(self, status_code):
         """
@@ -140,14 +158,14 @@ class HTTP:
         Fails if the response does not have a header named 'header_name'
         """
         assert header_name in self.response.headers,\
-               'Response did not have "%s" header, but should have' % header_name
+               'Response did not have "%s" header, but should have.' % header_name
 
     def response_should_not_have_header(self, header_name):
         """
         Fails if the response does has a header named 'header_name'
         """
         assert not header_name in self.response.headers,\
-               'Response did have "%s" header, but should not have' % header_name
+               'Response did have "%s" header, but should not have.' % header_name
 
     def get_response_header(self, header_name):
         """
@@ -163,7 +181,7 @@ class HTTP:
         self.response_should_have_header(header_name)
         actual = self.response.headers[header_name]
         assert actual == expected,\
-               'Response header "%s" should have been "%s" but was "%s"' % (
+               'Response header "%s" should have been "%s" but was "%s".' % (
                     header_name, expected, actual)
 
     def response_header_should_not_equal(self, header_name, not_expected):
@@ -195,9 +213,10 @@ class HTTP:
         """
         Set HTTP Basic Auth for next request.
 
-        See <a href="http://en.wikipedia.org/wiki/Basic_access_authentication">Wikipedia on "Basic access authentication"</a>.
+        See http://en.wikipedia.org/wiki/Basic_access_authentication
 
         'username' is the username to authenticate with, e.g. 'Aladdin'
+
         'password' is the password to use, e.g. 'open sesame'
         """
         self.set_request_header("Authorization", "Basic %s" % b64encode("%s:%s" % (username, password)))
@@ -217,6 +236,42 @@ class HTTP:
     def log_response_body(self, log_level='INFO'):
         LOGGER.write(self.response.body, log_level)
 
+    # json
+
+    def is_valid_json(self, json_string):
+        """
+        Attempts to parse `json_string` as JSON. Fails, if `json_string` is invalid JSON.
+
+        Example:
+        | Is Valid Json | {"foo": "bar"} |
+        """
+        json.loads(json_string)
+
+    @_with_json
+    def get_json_value(self, json_string, json_pointer):
+        """
+        Get the target node of the JSON document `json_string` specified by `json_pointer`.
+
+        Example:
+        | ${result}=       | Get Json Value   | {"foo": {"bar": [1,2,3]}} | /foo/bar |
+        | Should Be Equal  | ${result}        | [1, 2, 3]                 |          |
+        """
+        return jsonpointer.resolve_pointer(json_string, json_pointer)
+
+    @_with_json
+    def set_json_value(self, json_string, json_pointer, json_value):
+        """
+        Set the target node of the JSON document `json_string` specified by
+        JSON Pointer `json_pointer` to `json_value`.
+
+        Example:
+        | ${result}=       | Set Json Value | {"foo": {"bar": [1,2,3]}} | /foo | 12 |
+        | Should Be Equal  | ${result}      | {"foo": 12}               |      |    |
+        """
+        value = json.loads(json_value)
+        p = jsonpointer.set_pointer(json_string, json_pointer, value)
+        return json_string
+
     # debug
 
     def show_response_body_in_browser(self):
